"use strict";
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0
//
// Utilities for fetching the JWKS URI, to get the public keys with which to verify JWTs
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleFetcher = exports.fetch = void 0;
const error_js_1 = require("./error.js");
const _node_web_compat_1 = require("#node-web-compat");
/**
 * Execute a HTTPS request
 * @param uri - The URI
 * @param requestOptions - The RequestOptions to use (depending on the runtime context, either Node.js RequestOptions or Web Fetch init)
 * @param data - Data to send to the URI (e.g. POST data)
 * @returns - The response body as ArrayBuffer
 */
exports.fetch = _node_web_compat_1.nodeWebCompat.fetch.bind(undefined);
/**
 * HTTPS Fetcher. The fetch implementation has 1 immediate retry in case of errors
 *
 * @param defaultRequestOptions - The default RequestOptions to use on individual HTTPS requests
 *   (depending on the runtime context, either Node.js RequestOptions or Web Fetch init)
 */
class SimpleFetcher {
    constructor(props) {
        this.defaultRequestOptions = {
            timeout: _node_web_compat_1.nodeWebCompat.defaultFetchTimeouts.socketIdle,
            responseTimeout: _node_web_compat_1.nodeWebCompat.defaultFetchTimeouts.response,
            ...props?.defaultRequestOptions,
        };
    }
    /**
     * Execute a HTTPS request (with 1 immediate retry in case of errors)
     * @param uri - The URI
     * @param requestOptions - The RequestOptions to use (depending on the runtime context, either Node.js RequestOptions or Web Fetch init)
     * @param data - Data to send to the URI (e.g. POST data)
     * @returns - The response body as ArrayBuffer
     */
    async fetch(uri, requestOptions, data) {
        requestOptions = { ...this.defaultRequestOptions, ...requestOptions };
        try {
            return await (0, exports.fetch)(uri, requestOptions, data);
        }
        catch (err) {
            if (err instanceof error_js_1.NonRetryableFetchError) {
                throw err;
            }
            // Retry once, immediately
            return (0, exports.fetch)(uri, requestOptions, data);
        }
    }
}
exports.SimpleFetcher = SimpleFetcher;
